library('tidyverse')
library('dplyr')
library('readxl')
library('broom')
library('rlang')

rm(list=ls())

#Load data

rel_average <- read_excel("Average relativities.xlsx", range = cell_rows(1:10))   #average of State Treasury relativity forcasts. RELCODE 'A' = Average relativity
rel_longterm <- read_excel("Long-term relativities.xlsx", range = cell_rows(1:10))#relativities returning to their long-run trend at the end of the forward estaimte period. RELCODE 'L' = Long-run relativity
rel_current <- read_excel("Current relativities.xlsx")                            #relativities remain at their 2018-19 level over projection period. RELCODE 'C' = Current relativity

pop_MYEFO <- read_excel("MYEFO populations.xlsx")                                 #state populations. POPCODE 'M' = MYEFO

pool_central <- read_excel("Central GST pool.xlsx")                               #GST pool. POOLCODE 'C' = central case

state_rev <- read_excel("state_rev.xlsx")                                         #historical State revenue, and State revenue forecast in mid-year updates
gst_payments <- read_excel("gst_payments.xlsx")                                   #historical GST payments to each State. GST for forward estimates is projected below.

#Make everything long

pool_long <- pool_central %>% gather(key = STATE, value = POOL,NSW,VIC,QLD,WA,SA,TAS,ACT,NT) #create State-value pairs
pool_long <- pool_long[,c("YEAR","STATE","POOL","POOLCODE")]                                 #rearrange so that year and State are the leading columns

pop_long <- pop_MYEFO %>% gather(key = STATE, value = POP,NSW,VIC,QLD,WA,SA,TAS,ACT,NT)      #create State-value pairs
pop_long <- pop_long[,c("YEAR","STATE","POP","POPCODE","TOTAL")]                             #rearrange so that year and State are the leading columns

rel_long <- rbind.data.frame(rel_average,rel_longterm,rel_current)                           #bind all relativity data together
rel_long <- rel_long %>% gather(key = STATE, value = REL,NSW,VIC,QLD,WA,SA,TAS,ACT,NT)       #create State-value pairs
rel_long <- rel_long[,c("YEAR","STATE","REL","RELCODE")]                                     #rearrange so that year and State are the leading columns

#Combine data into single dataframe

data <- left_join(rel_long,pop_long,by = c("STATE","YEAR"))
data <- left_join(data,pool_long,by = c("STATE","YEAR"))

#### GST UNDER CURRENT STANDARD ####

data  <- data %>% group_by(RELCODE, POPCODE, POOLCODE, YEAR) %>% mutate(
  TOTAL = sum(POP),
  POPSHARE = POP/TOTAL, 
  ADJPOP = POP*REL,
  ADJPOPTOT = sum(ADJPOP),
  ADJPOPSHARE = ADJPOP/ADJPOPTOT,
  BASELINE = ADJPOPSHARE*POOL,
  BASELINE_PER_CAPITA = BASELINE/POP, 
  EPC = POOL/TOTAL,
  EFFECTIVE_REL = BASELINE_PER_CAPITA/EPC) %>% ungroup

#### PROJECT NON_GST REVENUE ####

#Prepare data 

gst_payments[gst_payments$YEAR %in% 2018:2020,] <- data %>% filter(YEAR %in% 2018:2020, RELCODE == "A") %>% select(STATE,BASELINE,YEAR) %>% 
                                                   spread(STATE,BASELINE) %>% select(YEAR,NSW,VIC,QLD,WA,SA,TAS,ACT,NT)    #add projected GST payments over the forward estimates to list of historical GST payments

revenue_nonGST <- state_rev - gst_payments    #each State's non-GST revenue
revenue_nonGST[,"YEAR"] <- 2000:2020
revenue_nonGST <- revenue_nonGST[-(1:8),]     #drop first 8 years (consider revenue from 2008-09 onwards)

#Project non-GST revenue

lm.df <- revenue_nonGST %>% gather( "STATE" , "VALUE" , 2:9) %>% group_by(STATE) %>% do( tidy( lm( VALUE ~ YEAR, data=.) ) ) %>% select( 1:3 ) %>% spread( term, estimate ) %>% setNames( c("STATE","A","B") )
proj_GST <- expand.grid( YEAR = 2021:2026 , STATE = c("NSW","VIC","QLD","WA","SA","TAS","ACT","NT")) %>% left_join( lm.df, by="STATE" ) %>% mutate( VALUE = A + B*YEAR ) %>% select( -A, -B ) %>% spread( STATE, VALUE ) %>% select(YEAR,NSW,VIC,QLD,WA,SA,TAS,ACT,NT)
revenue_nonGST <- rbind(revenue_nonGST[11:13,],proj_GST)

#combine with existing dataframe

rev_long <- revenue_nonGST %>% gather(key = STATE, value = REV_NONGST,NSW,VIC,QLD,WA,SA,TAS,ACT,NT) #create State-value pairs
rev_long <- rev_long[,c("YEAR","STATE","REV_NONGST")]                                               #rearrange so that year and State are the leading columns
data <- left_join(data,rev_long,by = c("STATE","YEAR"))                                             #join to existing data frame

#### GST UNDER NEW STANDARD ####

#set equalisation standard

standard = 1  #standard to equalise to. Set to 1 for equilisation to the average

#Calcualte GST under new standard

data  <- data %>% group_by(RELCODE, YEAR) %>% mutate(ABOVE_STANDARD = ifelse(EFFECTIVE_REL > standard, 1,0),
                  STANDARD_DEFICIT = ABOVE_STANDARD*(EFFECTIVE_REL-standard),
                  EPC_CONTRIBUTION = (1-ABOVE_STANDARD*STANDARD_DEFICIT)*POPSHARE, 
                  SUM_EPC_CONTRIBUTION = sum(EPC_CONTRIBUTION), 
                  PER_CAPITA_ETA = EPC*(STANDARD_DEFICIT+SUM_EPC_CONTRIBUTION),
                  GST_TOTAL_ETA = PER_CAPITA_ETA * POP,
                  REL_ETA = PER_CAPITA_ETA/EPC,
                  FOUR_YEAR = ifelse(YEAR <= 2022, (YEAR - 2018)*0.25, 1),
                  EIGHT_YEAR = ifelse(YEAR <= 2026, (YEAR - 2018)*0.125, 1)) %>%
                  gather( key=POLICY, value=X_YEAR_WEIGHT, FOUR_YEAR, EIGHT_YEAR)

#Calcualte variables of interest for Appendix F

data <- data %>% group_by(RELCODE, STATE, POLICY) %>% mutate(
                 GST_X_YEAR = BASELINE*(1-X_YEAR_WEIGHT) + GST_TOTAL_ETA*X_YEAR_WEIGHT,
                 TOTAL_REV_X_YEAR = REV_NONGST + BASELINE,
                 GST_PER_CAPITA_X_YEAR = GST_X_YEAR/POP,
                 REL_ETA_X_YEAR = GST_PER_CAPITA_X_YEAR/EPC,
                 CHANGE_GST_X_YEAR = GST_X_YEAR - BASELINE,
                 PER_CAPITA_CHANGE_GST_X_YEAR = GST_PER_CAPITA_X_YEAR - BASELINE_PER_CAPITA,
                 RATE_OF_CHANGE_X_YEAR  = CHANGE_GST_X_YEAR  - lag(CHANGE_GST_X_YEAR, 1),
                 PERCENT_CHANGE_X_YEAR = (CHANGE_GST_X_YEAR / TOTAL_REV_X_YEAR),
                 PERCENT_RATE_OF_CHANGE_X_YEAR = (RATE_OF_CHANGE_X_YEAR / TOTAL_REV_X_YEAR)) %>% ungroup

rm(list=setdiff(ls(), "data"))

#### ANALYSIS ####

#Extract tables of the desired data

export_table <- data %>% filter(RELCODE == "C", POLICY == "FOUR_YEAR") %>% select(YEAR,STATE,PER_CAPITA_CHANGE_GST_X_YEAR)
export_table <- spread(export_table,STATE,PER_CAPITA_CHANGE_GST_X_YEAR)
export_table <- export_table[,c("YEAR","NSW","VIC","QLD","WA","SA","TAS","ACT","NT")]
